@@ -1,24 +1,9 @@
 """AMQP Channels."""
 # Copyright (C) 2007-2008 Barry Pederson <bp@barryp.org>
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 from __future__ import absolute_import, unicode_literals
 
 import logging
 import socket
-
 from collections import defaultdict
 from warnings import warn
 
@@ -26,10 +11,9 @@ from vine import ensure_promise
 
 from . import spec
 from .abstract_channel import AbstractChannel
-from .exceptions import (
-    ChannelError, ConsumerCancelled,
-    RecoverableChannelError, RecoverableConnectionError, error_for_code,
-)
+from .exceptions import (ChannelError, ConsumerCancelled,
+                         RecoverableChannelError, RecoverableConnectionError,
+                         error_for_code, MessageNacked)
 from .five import Queue
 from .protocol import queue_declare_ok_t
 
@@ -109,6 +93,7 @@ class Channel(AbstractChannel):
         spec.method(spec.Tx.SelectOk),
         spec.method(spec.Confirm.SelectOk),
         spec.method(spec.Basic.Ack, 'Lb'),
+        spec.method(spec.Basic.Nack, 'Lb'),
     }
     _METHODS = {m.method_sig: m for m in _METHODS}
 
@@ -154,6 +139,7 @@ class Channel(AbstractChannel):
             spec.Basic.Deliver: self._on_basic_deliver,
             spec.Basic.Return: self._on_basic_return,
             spec.Basic.Ack: self._on_basic_ack,
+            spec.Basic.Nack: self._on_basic_nack,
         })
 
     def collect(self):
@@ -233,12 +219,14 @@ class Channel(AbstractChannel):
             if is_closed:
                 return
 
+            self.is_closing = True
             return self.send_method(
                 spec.Channel.Close, argsig,
                 (reply_code, reply_text, method_sig[0], method_sig[1]),
                 wait=spec.Channel.CloseOk,
             )
         finally:
+            self.is_closing = False
             self.connection = None
 
     def _on_close(self, reply_code, reply_text, class_id, method_id):
@@ -288,10 +276,11 @@ class Channel(AbstractChannel):
                 is the ID of the method.
         """
         self.send_method(spec.Channel.CloseOk)
-        self._do_revive()
-        raise error_for_code(
-            reply_code, reply_text, (class_id, method_id), ChannelError,
-        )
+        if not self.connection.is_closing:
+            self._do_revive()
+            raise error_for_code(
+                reply_code, reply_text, (class_id, method_id), ChannelError,
+            )
 
     def _on_close_ok(self):
         """Confirm a channel close.
@@ -1151,8 +1140,8 @@ class Channel(AbstractChannel):
                 True.
 
         Returns a tuple containing 3 items:
-            the name of the queue (essential for automatically-named queues)
-            message count
+            the name of the queue (essential for automatically-named queues),
+            message count and
             consumer count
         """
         self.send_method(
@@ -1231,6 +1220,8 @@ class Channel(AbstractChannel):
                 client should not wait for a reply method.  If the
                 server could not complete the method it will raise a
                 channel or connection exception.
+
+        If nowait is False, returns the number of deleted messages.
         """
         return self.send_method(
             spec.Queue.Delete, argsig,
@@ -1291,7 +1282,7 @@ class Channel(AbstractChannel):
                 server could not complete the method it will raise a
                 channel or connection exception.
 
-        if nowait is False, returns a message_count
+        If nowait is False, returns a number of purged messages.
         """
         return self.send_method(
             spec.Queue.Purge, argsig, (0, queue, nowait),
@@ -1573,14 +1564,26 @@ class Channel(AbstractChannel):
         """
         p = self.send_method(
             spec.Basic.Consume, argsig,
-            (0, queue, consumer_tag, no_local, no_ack, exclusive,
-             nowait, arguments),
+            (
+                0, queue, consumer_tag, no_local, no_ack, exclusive,
+                nowait, arguments
+            ),
             wait=None if nowait else spec.Basic.ConsumeOk,
+            returns_tuple=True
         )
 
-        # XXX Fix this hack
-        if not nowait and not consumer_tag:
-            consumer_tag = p
+        if not nowait:
+            # send_method() returns (consumer_tag,) tuple.
+            # consumer_tag is returned by broker using following rules:
+            # * consumer_tag is not specified by client, random one
+            #   is generated by Broker
+            # * consumer_tag is provided by client, the same one
+            #   is returned by broker
+            consumer_tag = p[0]
+        elif nowait and not consumer_tag:
+            raise ValueError(
+                'Consumer tag must be specified when nowait is True'
+            )
 
         self.callbacks[consumer_tag] = callback
 
@@ -1588,7 +1591,11 @@ class Channel(AbstractChannel):
             self.cancel_callbacks[consumer_tag] = on_cancel
         if no_ack:
             self.no_ack_consumers.add(consumer_tag)
-        return p
+
+        if not nowait:
+            return consumer_tag
+        else:
+            return p
 
     def _on_basic_deliver(self, consumer_tag, delivery_tag, redelivered,
                           exchange, routing_key, msg):
@@ -1604,7 +1611,7 @@ class Channel(AbstractChannel):
         try:
             fun = self.callbacks[consumer_tag]
         except KeyError:
-            AMQP_LOGGER.warn(
+            AMQP_LOGGER.warning(
                 REJECTED_MESSAGE_WITHOUT_CALLBACK,
                 delivery_tag, consumer_tag, exchange, routing_key,
             )
@@ -1646,7 +1653,8 @@ class Channel(AbstractChannel):
                 reliability.  Messages can get lost if a client dies
                 before it can deliver them to the application.
 
-        Non-blocking, returns a message object, or None.
+        Non-blocking, returns a amqp.basic_message.Message object,
+        or None if queue is empty.
         """
         ret = self.send_method(
             spec.Basic.Get, argsig, (0, queue, no_ack),
@@ -1681,6 +1689,11 @@ class Channel(AbstractChannel):
         configuration and distributed to any active consumers when the
         transaction, if any, is committed.
 
+        When channel is in confirm mode (when Connection parameter
+        confirm_publish is set to True), each message is confirmed. When
+        broker rejects published message (e.g. due internal broker
+        constrains), MessageNacked exception is raised.
+
         PARAMETERS:
             exchange: shortstr
 
@@ -1741,6 +1754,16 @@ class Channel(AbstractChannel):
         if not self.connection:
             raise RecoverableConnectionError(
                 'basic_publish: connection closed')
+
+        capabilities = self.connection.\
+            client_properties.get('capabilities', {})
+        if capabilities.get('connection.blocked', False):
+            try:
+                # Check if an event was sent, such as the out of memory message
+                self.connection.drain_events(timeout=0)
+            except socket.timeout:
+                pass
+
         try:
             with self.connection.transport.having_timeout(timeout):
                 return self.send_method(
@@ -1752,11 +1775,18 @@ class Channel(AbstractChannel):
     basic_publish = _basic_publish
 
     def basic_publish_confirm(self, *args, **kwargs):
+
+        def confirm_handler(method, *args):
+            # When RMQ nacks message we are raising MessageNacked exception
+            if method == spec.Basic.Nack:
+                raise MessageNacked()
+
         if not self._confirm_selected:
             self._confirm_selected = True
             self.confirm_select()
         ret = self._basic_publish(*args, **kwargs)
-        self.wait(spec.Basic.Ack)
+        # Waiting for confirmation of message.
+        self.wait([spec.Basic.Ack, spec.Basic.Nack], callback=confirm_handler)
         return ret
 
     def basic_qos(self, prefetch_size, prefetch_count, a_global,
@@ -2053,3 +2083,7 @@ class Channel(AbstractChannel):
     def _on_basic_ack(self, delivery_tag, multiple):
         for callback in self.events['basic_ack']:
             callback(delivery_tag, multiple)
+
+    def _on_basic_nack(self, delivery_tag, multiple):
+        for callback in self.events['basic_nack']:
+            callback(delivery_tag, multiple)
