@@ -2,15 +2,12 @@
 from __future__ import absolute_import, unicode_literals
 
 import logging
-import sys
 
 # enables celery 3.1.23 to start again
-from vine import promise                # noqa
+from vine import promise  # noqa
 from vine.utils import wraps
 
-from .five import string_t
-
-is_py3k = sys.version_info[0] == 3
+from .five import PY3, string_t, text_t
 
 try:
     import fcntl
@@ -68,25 +65,25 @@ def coro(gen):
     return _boot
 
 
-if is_py3k:  # pragma: no cover
+if PY3:  # pragma: no cover
 
     def str_to_bytes(s):
         """Convert str to bytes."""
         if isinstance(s, str):
-            return s.encode()
+            return s.encode('utf-8', 'surrogatepass')
         return s
 
     def bytes_to_str(s):
         """Convert bytes to str."""
         if isinstance(s, bytes):
-            return s.decode()
+            return s.decode('utf-8', 'surrogatepass')
         return s
 else:
 
     def str_to_bytes(s):                # noqa
         """Convert str to bytes."""
-        if isinstance(s, unicode):
-            return s.encode()
+        if isinstance(s, text_t):
+            return s.encode('utf-8')
         return s
 
     def bytes_to_str(s):                # noqa
